@echo off
set HOME=%HOMEDRIVE%%HOMEPATH%
set GAZEBO_MASTER_URI=http://localhost:11345
set GAZEBO_MODEL_DATABASE_URI=http://models.gazebosim.org
set GAZEBO_RESOURCE_PATH=D:\bld\gazebo_1624746865310\_h_env\Library\share\gazebo-11;%GAZEBO_RESOURCE_PATH%
set GAZEBO_PLUGIN_PATH=D:\bld\gazebo_1624746865310\_h_env\Library\bin\gazebo-11\plugins;%GAZEBO_PLUGIN_PATH%
set GAZEBO_MODEL_PATH=D:\bld\gazebo_1624746865310\_h_env\Library\share\gazebo-11\models;%GAZEBO_MODEL_PATH%
set LD_LIBRARY_PATH=%LD_LIBRARY_PATH%;D:\bld\gazebo_1624746865310\_h_env\Library\bin\gazebo-11\plugins
set OGRE_RESOURCE_PATH=D:\bld\gazebo_1624746865310\_h_env\Library\bin
set PATH=%LD_LIBRARY_PATH%;%PATH%
