// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: visual.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_visual_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_visual_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3016000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3016000 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/generated_enum_reflection.h>
#include <google/protobuf/unknown_field_set.h>
#include "pose.pb.h"
#include "geometry.pb.h"
#include "material.pb.h"
#include "plugin.pb.h"
#include "vector3d.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_visual_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_visual_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxiliaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[2]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_visual_2eproto;
namespace gazebo {
namespace msgs {
class Visual;
struct VisualDefaultTypeInternal;
GZ_MSGS_VISIBLE extern VisualDefaultTypeInternal _Visual_default_instance_;
class Visual_Meta;
struct Visual_MetaDefaultTypeInternal;
GZ_MSGS_VISIBLE extern Visual_MetaDefaultTypeInternal _Visual_Meta_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::Visual* Arena::CreateMaybeMessage<::gazebo::msgs::Visual>(Arena*);
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::Visual_Meta* Arena::CreateMaybeMessage<::gazebo::msgs::Visual_Meta>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

enum Visual_Type : int {
  Visual_Type_ENTITY = 0,
  Visual_Type_MODEL = 1,
  Visual_Type_LINK = 2,
  Visual_Type_VISUAL = 3,
  Visual_Type_COLLISION = 4,
  Visual_Type_SENSOR = 5,
  Visual_Type_GUI = 6,
  Visual_Type_PHYSICS = 7
};
GZ_MSGS_VISIBLE bool Visual_Type_IsValid(int value);
constexpr Visual_Type Visual_Type_Type_MIN = Visual_Type_ENTITY;
constexpr Visual_Type Visual_Type_Type_MAX = Visual_Type_PHYSICS;
constexpr int Visual_Type_Type_ARRAYSIZE = Visual_Type_Type_MAX + 1;

GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* Visual_Type_descriptor();
template<typename T>
inline const std::string& Visual_Type_Name(T enum_t_value) {
  static_assert(::std::is_same<T, Visual_Type>::value ||
    ::std::is_integral<T>::value,
    "Incorrect type passed to function Visual_Type_Name.");
  return ::PROTOBUF_NAMESPACE_ID::internal::NameOfEnum(
    Visual_Type_descriptor(), enum_t_value);
}
inline bool Visual_Type_Parse(
    ::PROTOBUF_NAMESPACE_ID::ConstStringParam name, Visual_Type* value) {
  return ::PROTOBUF_NAMESPACE_ID::internal::ParseNamedEnum<Visual_Type>(
    Visual_Type_descriptor(), name, value);
}
// ===================================================================

class GZ_MSGS_VISIBLE Visual_Meta PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.Visual.Meta) */ {
 public:
  inline Visual_Meta() : Visual_Meta(nullptr) {}
  ~Visual_Meta() override;
  explicit constexpr Visual_Meta(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Visual_Meta(const Visual_Meta& from);
  Visual_Meta(Visual_Meta&& from) noexcept
    : Visual_Meta() {
    *this = ::std::move(from);
  }

  inline Visual_Meta& operator=(const Visual_Meta& from) {
    CopyFrom(from);
    return *this;
  }
  inline Visual_Meta& operator=(Visual_Meta&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Visual_Meta& default_instance() {
    return *internal_default_instance();
  }
  static inline const Visual_Meta* internal_default_instance() {
    return reinterpret_cast<const Visual_Meta*>(
               &_Visual_Meta_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Visual_Meta& a, Visual_Meta& b) {
    a.Swap(&b);
  }
  inline void Swap(Visual_Meta* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Visual_Meta* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline Visual_Meta* New() const final {
    return CreateMaybeMessage<Visual_Meta>(nullptr);
  }

  Visual_Meta* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<Visual_Meta>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const Visual_Meta& from);
  void MergeFrom(const Visual_Meta& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Visual_Meta* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.Visual.Meta";
  }
  protected:
  explicit Visual_Meta(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kLayerFieldNumber = 1,
  };
  // optional int32 layer = 1;
  bool has_layer() const;
  private:
  bool _internal_has_layer() const;
  public:
  void clear_layer();
  ::PROTOBUF_NAMESPACE_ID::int32 layer() const;
  void set_layer(::PROTOBUF_NAMESPACE_ID::int32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::int32 _internal_layer() const;
  void _internal_set_layer(::PROTOBUF_NAMESPACE_ID::int32 value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.Visual.Meta)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::int32 layer_;
  friend struct ::TableStruct_visual_2eproto;
};
// -------------------------------------------------------------------

class GZ_MSGS_VISIBLE Visual PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.Visual) */ {
 public:
  inline Visual() : Visual(nullptr) {}
  ~Visual() override;
  explicit constexpr Visual(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Visual(const Visual& from);
  Visual(Visual&& from) noexcept
    : Visual() {
    *this = ::std::move(from);
  }

  inline Visual& operator=(const Visual& from) {
    CopyFrom(from);
    return *this;
  }
  inline Visual& operator=(Visual&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Visual& default_instance() {
    return *internal_default_instance();
  }
  static inline const Visual* internal_default_instance() {
    return reinterpret_cast<const Visual*>(
               &_Visual_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(Visual& a, Visual& b) {
    a.Swap(&b);
  }
  inline void Swap(Visual* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Visual* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline Visual* New() const final {
    return CreateMaybeMessage<Visual>(nullptr);
  }

  Visual* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<Visual>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const Visual& from);
  void MergeFrom(const Visual& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Visual* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.Visual";
  }
  protected:
  explicit Visual(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef Visual_Meta Meta;

  typedef Visual_Type Type;
  static constexpr Type ENTITY =
    Visual_Type_ENTITY;
  static constexpr Type MODEL =
    Visual_Type_MODEL;
  static constexpr Type LINK =
    Visual_Type_LINK;
  static constexpr Type VISUAL =
    Visual_Type_VISUAL;
  static constexpr Type COLLISION =
    Visual_Type_COLLISION;
  static constexpr Type SENSOR =
    Visual_Type_SENSOR;
  static constexpr Type GUI =
    Visual_Type_GUI;
  static constexpr Type PHYSICS =
    Visual_Type_PHYSICS;
  static inline bool Type_IsValid(int value) {
    return Visual_Type_IsValid(value);
  }
  static constexpr Type Type_MIN =
    Visual_Type_Type_MIN;
  static constexpr Type Type_MAX =
    Visual_Type_Type_MAX;
  static constexpr int Type_ARRAYSIZE =
    Visual_Type_Type_ARRAYSIZE;
  static inline const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor*
  Type_descriptor() {
    return Visual_Type_descriptor();
  }
  template<typename T>
  static inline const std::string& Type_Name(T enum_t_value) {
    static_assert(::std::is_same<T, Type>::value ||
      ::std::is_integral<T>::value,
      "Incorrect type passed to function Type_Name.");
    return Visual_Type_Name(enum_t_value);
  }
  static inline bool Type_Parse(::PROTOBUF_NAMESPACE_ID::ConstStringParam name,
      Type* value) {
    return Visual_Type_Parse(name, value);
  }

  // accessors -------------------------------------------------------

  enum : int {
    kPluginFieldNumber = 14,
    kNameFieldNumber = 1,
    kParentNameFieldNumber = 3,
    kPoseFieldNumber = 8,
    kGeometryFieldNumber = 9,
    kMaterialFieldNumber = 10,
    kScaleFieldNumber = 15,
    kMetaFieldNumber = 16,
    kIdFieldNumber = 2,
    kParentIdFieldNumber = 4,
    kTransparencyFieldNumber = 6,
    kLaserRetroFieldNumber = 7,
    kCastShadowsFieldNumber = 5,
    kVisibleFieldNumber = 11,
    kDeleteMeFieldNumber = 12,
    kIsStaticFieldNumber = 13,
    kTypeFieldNumber = 17,
  };
  // repeated .gazebo.msgs.Plugin plugin = 14;
  int plugin_size() const;
  private:
  int _internal_plugin_size() const;
  public:
  void clear_plugin();
  ::gazebo::msgs::Plugin* mutable_plugin(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Plugin >*
      mutable_plugin();
  private:
  const ::gazebo::msgs::Plugin& _internal_plugin(int index) const;
  ::gazebo::msgs::Plugin* _internal_add_plugin();
  public:
  const ::gazebo::msgs::Plugin& plugin(int index) const;
  ::gazebo::msgs::Plugin* add_plugin();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Plugin >&
      plugin() const;

  // required string name = 1;
  bool has_name() const;
  private:
  bool _internal_has_name() const;
  public:
  void clear_name();
  const std::string& name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_name();
  std::string* release_name();
  void set_allocated_name(std::string* name);
  private:
  const std::string& _internal_name() const;
  void _internal_set_name(const std::string& value);
  std::string* _internal_mutable_name();
  public:

  // required string parent_name = 3;
  bool has_parent_name() const;
  private:
  bool _internal_has_parent_name() const;
  public:
  void clear_parent_name();
  const std::string& parent_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_parent_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_parent_name();
  std::string* release_parent_name();
  void set_allocated_parent_name(std::string* parent_name);
  private:
  const std::string& _internal_parent_name() const;
  void _internal_set_parent_name(const std::string& value);
  std::string* _internal_mutable_parent_name();
  public:

  // optional .gazebo.msgs.Pose pose = 8;
  bool has_pose() const;
  private:
  bool _internal_has_pose() const;
  public:
  void clear_pose();
  const ::gazebo::msgs::Pose& pose() const;
  ::gazebo::msgs::Pose* release_pose();
  ::gazebo::msgs::Pose* mutable_pose();
  void set_allocated_pose(::gazebo::msgs::Pose* pose);
  private:
  const ::gazebo::msgs::Pose& _internal_pose() const;
  ::gazebo::msgs::Pose* _internal_mutable_pose();
  public:
  void unsafe_arena_set_allocated_pose(
      ::gazebo::msgs::Pose* pose);
  ::gazebo::msgs::Pose* unsafe_arena_release_pose();

  // optional .gazebo.msgs.Geometry geometry = 9;
  bool has_geometry() const;
  private:
  bool _internal_has_geometry() const;
  public:
  void clear_geometry();
  const ::gazebo::msgs::Geometry& geometry() const;
  ::gazebo::msgs::Geometry* release_geometry();
  ::gazebo::msgs::Geometry* mutable_geometry();
  void set_allocated_geometry(::gazebo::msgs::Geometry* geometry);
  private:
  const ::gazebo::msgs::Geometry& _internal_geometry() const;
  ::gazebo::msgs::Geometry* _internal_mutable_geometry();
  public:
  void unsafe_arena_set_allocated_geometry(
      ::gazebo::msgs::Geometry* geometry);
  ::gazebo::msgs::Geometry* unsafe_arena_release_geometry();

  // optional .gazebo.msgs.Material material = 10;
  bool has_material() const;
  private:
  bool _internal_has_material() const;
  public:
  void clear_material();
  const ::gazebo::msgs::Material& material() const;
  ::gazebo::msgs::Material* release_material();
  ::gazebo::msgs::Material* mutable_material();
  void set_allocated_material(::gazebo::msgs::Material* material);
  private:
  const ::gazebo::msgs::Material& _internal_material() const;
  ::gazebo::msgs::Material* _internal_mutable_material();
  public:
  void unsafe_arena_set_allocated_material(
      ::gazebo::msgs::Material* material);
  ::gazebo::msgs::Material* unsafe_arena_release_material();

  // optional .gazebo.msgs.Vector3d scale = 15;
  bool has_scale() const;
  private:
  bool _internal_has_scale() const;
  public:
  void clear_scale();
  const ::gazebo::msgs::Vector3d& scale() const;
  ::gazebo::msgs::Vector3d* release_scale();
  ::gazebo::msgs::Vector3d* mutable_scale();
  void set_allocated_scale(::gazebo::msgs::Vector3d* scale);
  private:
  const ::gazebo::msgs::Vector3d& _internal_scale() const;
  ::gazebo::msgs::Vector3d* _internal_mutable_scale();
  public:
  void unsafe_arena_set_allocated_scale(
      ::gazebo::msgs::Vector3d* scale);
  ::gazebo::msgs::Vector3d* unsafe_arena_release_scale();

  // optional .gazebo.msgs.Visual.Meta meta = 16;
  bool has_meta() const;
  private:
  bool _internal_has_meta() const;
  public:
  void clear_meta();
  const ::gazebo::msgs::Visual_Meta& meta() const;
  ::gazebo::msgs::Visual_Meta* release_meta();
  ::gazebo::msgs::Visual_Meta* mutable_meta();
  void set_allocated_meta(::gazebo::msgs::Visual_Meta* meta);
  private:
  const ::gazebo::msgs::Visual_Meta& _internal_meta() const;
  ::gazebo::msgs::Visual_Meta* _internal_mutable_meta();
  public:
  void unsafe_arena_set_allocated_meta(
      ::gazebo::msgs::Visual_Meta* meta);
  ::gazebo::msgs::Visual_Meta* unsafe_arena_release_meta();

  // optional uint32 id = 2;
  bool has_id() const;
  private:
  bool _internal_has_id() const;
  public:
  void clear_id();
  ::PROTOBUF_NAMESPACE_ID::uint32 id() const;
  void set_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::uint32 _internal_id() const;
  void _internal_set_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  public:

  // optional uint32 parent_id = 4;
  bool has_parent_id() const;
  private:
  bool _internal_has_parent_id() const;
  public:
  void clear_parent_id();
  ::PROTOBUF_NAMESPACE_ID::uint32 parent_id() const;
  void set_parent_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  private:
  ::PROTOBUF_NAMESPACE_ID::uint32 _internal_parent_id() const;
  void _internal_set_parent_id(::PROTOBUF_NAMESPACE_ID::uint32 value);
  public:

  // optional double transparency = 6;
  bool has_transparency() const;
  private:
  bool _internal_has_transparency() const;
  public:
  void clear_transparency();
  double transparency() const;
  void set_transparency(double value);
  private:
  double _internal_transparency() const;
  void _internal_set_transparency(double value);
  public:

  // optional double laser_retro = 7;
  bool has_laser_retro() const;
  private:
  bool _internal_has_laser_retro() const;
  public:
  void clear_laser_retro();
  double laser_retro() const;
  void set_laser_retro(double value);
  private:
  double _internal_laser_retro() const;
  void _internal_set_laser_retro(double value);
  public:

  // optional bool cast_shadows = 5;
  bool has_cast_shadows() const;
  private:
  bool _internal_has_cast_shadows() const;
  public:
  void clear_cast_shadows();
  bool cast_shadows() const;
  void set_cast_shadows(bool value);
  private:
  bool _internal_cast_shadows() const;
  void _internal_set_cast_shadows(bool value);
  public:

  // optional bool visible = 11;
  bool has_visible() const;
  private:
  bool _internal_has_visible() const;
  public:
  void clear_visible();
  bool visible() const;
  void set_visible(bool value);
  private:
  bool _internal_visible() const;
  void _internal_set_visible(bool value);
  public:

  // optional bool delete_me = 12;
  bool has_delete_me() const;
  private:
  bool _internal_has_delete_me() const;
  public:
  void clear_delete_me();
  bool delete_me() const;
  void set_delete_me(bool value);
  private:
  bool _internal_delete_me() const;
  void _internal_set_delete_me(bool value);
  public:

  // optional bool is_static = 13;
  bool has_is_static() const;
  private:
  bool _internal_has_is_static() const;
  public:
  void clear_is_static();
  bool is_static() const;
  void set_is_static(bool value);
  private:
  bool _internal_is_static() const;
  void _internal_set_is_static(bool value);
  public:

  // optional .gazebo.msgs.Visual.Type type = 17;
  bool has_type() const;
  private:
  bool _internal_has_type() const;
  public:
  void clear_type();
  ::gazebo::msgs::Visual_Type type() const;
  void set_type(::gazebo::msgs::Visual_Type value);
  private:
  ::gazebo::msgs::Visual_Type _internal_type() const;
  void _internal_set_type(::gazebo::msgs::Visual_Type value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.Visual)
 private:
  class _Internal;

  // helper for ByteSizeLong()
  size_t RequiredFieldsByteSizeFallback() const;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Plugin > plugin_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr name_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr parent_name_;
  ::gazebo::msgs::Pose* pose_;
  ::gazebo::msgs::Geometry* geometry_;
  ::gazebo::msgs::Material* material_;
  ::gazebo::msgs::Vector3d* scale_;
  ::gazebo::msgs::Visual_Meta* meta_;
  ::PROTOBUF_NAMESPACE_ID::uint32 id_;
  ::PROTOBUF_NAMESPACE_ID::uint32 parent_id_;
  double transparency_;
  double laser_retro_;
  bool cast_shadows_;
  bool visible_;
  bool delete_me_;
  bool is_static_;
  int type_;
  friend struct ::TableStruct_visual_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Visual_Meta

// optional int32 layer = 1;
inline bool Visual_Meta::_internal_has_layer() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Visual_Meta::has_layer() const {
  return _internal_has_layer();
}
inline void Visual_Meta::clear_layer() {
  layer_ = 0;
  _has_bits_[0] &= ~0x00000001u;
}
inline ::PROTOBUF_NAMESPACE_ID::int32 Visual_Meta::_internal_layer() const {
  return layer_;
}
inline ::PROTOBUF_NAMESPACE_ID::int32 Visual_Meta::layer() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.Meta.layer)
  return _internal_layer();
}
inline void Visual_Meta::_internal_set_layer(::PROTOBUF_NAMESPACE_ID::int32 value) {
  _has_bits_[0] |= 0x00000001u;
  layer_ = value;
}
inline void Visual_Meta::set_layer(::PROTOBUF_NAMESPACE_ID::int32 value) {
  _internal_set_layer(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.Meta.layer)
}

// -------------------------------------------------------------------

// Visual

// required string name = 1;
inline bool Visual::_internal_has_name() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Visual::has_name() const {
  return _internal_has_name();
}
inline void Visual::clear_name() {
  name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& Visual::name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.name)
  return _internal_name();
}
template <typename ArgT0, typename... ArgT>
PROTOBUF_ALWAYS_INLINE
inline void Visual::set_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArena());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.name)
}
inline std::string* Visual::mutable_name() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.name)
  return _internal_mutable_name();
}
inline const std::string& Visual::_internal_name() const {
  return name_.Get();
}
inline void Visual::_internal_set_name(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArena());
}
inline std::string* Visual::_internal_mutable_name() {
  _has_bits_[0] |= 0x00000001u;
  return name_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArena());
}
inline std::string* Visual::release_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.name)
  if (!_internal_has_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  return name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline void Visual::set_allocated_name(std::string* name) {
  if (name != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), name,
      GetArena());
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.name)
}

// optional uint32 id = 2;
inline bool Visual::_internal_has_id() const {
  bool value = (_has_bits_[0] & 0x00000080u) != 0;
  return value;
}
inline bool Visual::has_id() const {
  return _internal_has_id();
}
inline void Visual::clear_id() {
  id_ = 0u;
  _has_bits_[0] &= ~0x00000080u;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 Visual::_internal_id() const {
  return id_;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 Visual::id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.id)
  return _internal_id();
}
inline void Visual::_internal_set_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _has_bits_[0] |= 0x00000080u;
  id_ = value;
}
inline void Visual::set_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _internal_set_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.id)
}

// required string parent_name = 3;
inline bool Visual::_internal_has_parent_name() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool Visual::has_parent_name() const {
  return _internal_has_parent_name();
}
inline void Visual::clear_parent_name() {
  parent_name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000002u;
}
inline const std::string& Visual::parent_name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.parent_name)
  return _internal_parent_name();
}
template <typename ArgT0, typename... ArgT>
PROTOBUF_ALWAYS_INLINE
inline void Visual::set_parent_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000002u;
 parent_name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArena());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.parent_name)
}
inline std::string* Visual::mutable_parent_name() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.parent_name)
  return _internal_mutable_parent_name();
}
inline const std::string& Visual::_internal_parent_name() const {
  return parent_name_.Get();
}
inline void Visual::_internal_set_parent_name(const std::string& value) {
  _has_bits_[0] |= 0x00000002u;
  parent_name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArena());
}
inline std::string* Visual::_internal_mutable_parent_name() {
  _has_bits_[0] |= 0x00000002u;
  return parent_name_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArena());
}
inline std::string* Visual::release_parent_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.parent_name)
  if (!_internal_has_parent_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000002u;
  return parent_name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline void Visual::set_allocated_parent_name(std::string* parent_name) {
  if (parent_name != nullptr) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  parent_name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), parent_name,
      GetArena());
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.parent_name)
}

// optional uint32 parent_id = 4;
inline bool Visual::_internal_has_parent_id() const {
  bool value = (_has_bits_[0] & 0x00000100u) != 0;
  return value;
}
inline bool Visual::has_parent_id() const {
  return _internal_has_parent_id();
}
inline void Visual::clear_parent_id() {
  parent_id_ = 0u;
  _has_bits_[0] &= ~0x00000100u;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 Visual::_internal_parent_id() const {
  return parent_id_;
}
inline ::PROTOBUF_NAMESPACE_ID::uint32 Visual::parent_id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.parent_id)
  return _internal_parent_id();
}
inline void Visual::_internal_set_parent_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _has_bits_[0] |= 0x00000100u;
  parent_id_ = value;
}
inline void Visual::set_parent_id(::PROTOBUF_NAMESPACE_ID::uint32 value) {
  _internal_set_parent_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.parent_id)
}

// optional bool cast_shadows = 5;
inline bool Visual::_internal_has_cast_shadows() const {
  bool value = (_has_bits_[0] & 0x00000800u) != 0;
  return value;
}
inline bool Visual::has_cast_shadows() const {
  return _internal_has_cast_shadows();
}
inline void Visual::clear_cast_shadows() {
  cast_shadows_ = false;
  _has_bits_[0] &= ~0x00000800u;
}
inline bool Visual::_internal_cast_shadows() const {
  return cast_shadows_;
}
inline bool Visual::cast_shadows() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.cast_shadows)
  return _internal_cast_shadows();
}
inline void Visual::_internal_set_cast_shadows(bool value) {
  _has_bits_[0] |= 0x00000800u;
  cast_shadows_ = value;
}
inline void Visual::set_cast_shadows(bool value) {
  _internal_set_cast_shadows(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.cast_shadows)
}

// optional double transparency = 6;
inline bool Visual::_internal_has_transparency() const {
  bool value = (_has_bits_[0] & 0x00000200u) != 0;
  return value;
}
inline bool Visual::has_transparency() const {
  return _internal_has_transparency();
}
inline void Visual::clear_transparency() {
  transparency_ = 0;
  _has_bits_[0] &= ~0x00000200u;
}
inline double Visual::_internal_transparency() const {
  return transparency_;
}
inline double Visual::transparency() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.transparency)
  return _internal_transparency();
}
inline void Visual::_internal_set_transparency(double value) {
  _has_bits_[0] |= 0x00000200u;
  transparency_ = value;
}
inline void Visual::set_transparency(double value) {
  _internal_set_transparency(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.transparency)
}

// optional double laser_retro = 7;
inline bool Visual::_internal_has_laser_retro() const {
  bool value = (_has_bits_[0] & 0x00000400u) != 0;
  return value;
}
inline bool Visual::has_laser_retro() const {
  return _internal_has_laser_retro();
}
inline void Visual::clear_laser_retro() {
  laser_retro_ = 0;
  _has_bits_[0] &= ~0x00000400u;
}
inline double Visual::_internal_laser_retro() const {
  return laser_retro_;
}
inline double Visual::laser_retro() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.laser_retro)
  return _internal_laser_retro();
}
inline void Visual::_internal_set_laser_retro(double value) {
  _has_bits_[0] |= 0x00000400u;
  laser_retro_ = value;
}
inline void Visual::set_laser_retro(double value) {
  _internal_set_laser_retro(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.laser_retro)
}

// optional .gazebo.msgs.Pose pose = 8;
inline bool Visual::_internal_has_pose() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || pose_ != nullptr);
  return value;
}
inline bool Visual::has_pose() const {
  return _internal_has_pose();
}
inline const ::gazebo::msgs::Pose& Visual::_internal_pose() const {
  const ::gazebo::msgs::Pose* p = pose_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Pose&>(
      ::gazebo::msgs::_Pose_default_instance_);
}
inline const ::gazebo::msgs::Pose& Visual::pose() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.pose)
  return _internal_pose();
}
inline void Visual::unsafe_arena_set_allocated_pose(
    ::gazebo::msgs::Pose* pose) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  pose_ = pose;
  if (pose) {
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Visual.pose)
}
inline ::gazebo::msgs::Pose* Visual::release_pose() {
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Pose* Visual::unsafe_arena_release_pose() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.pose)
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Pose* Visual::_internal_mutable_pose() {
  _has_bits_[0] |= 0x00000004u;
  if (pose_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Pose>(GetArena());
    pose_ = p;
  }
  return pose_;
}
inline ::gazebo::msgs::Pose* Visual::mutable_pose() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.pose)
  return _internal_mutable_pose();
}
inline void Visual::set_allocated_pose(::gazebo::msgs::Pose* pose) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  if (pose) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose)->GetArena();
    if (message_arena != submessage_arena) {
      pose = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, pose, submessage_arena);
    }
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  pose_ = pose;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.pose)
}

// optional .gazebo.msgs.Geometry geometry = 9;
inline bool Visual::_internal_has_geometry() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || geometry_ != nullptr);
  return value;
}
inline bool Visual::has_geometry() const {
  return _internal_has_geometry();
}
inline const ::gazebo::msgs::Geometry& Visual::_internal_geometry() const {
  const ::gazebo::msgs::Geometry* p = geometry_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Geometry&>(
      ::gazebo::msgs::_Geometry_default_instance_);
}
inline const ::gazebo::msgs::Geometry& Visual::geometry() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.geometry)
  return _internal_geometry();
}
inline void Visual::unsafe_arena_set_allocated_geometry(
    ::gazebo::msgs::Geometry* geometry) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(geometry_);
  }
  geometry_ = geometry;
  if (geometry) {
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Visual.geometry)
}
inline ::gazebo::msgs::Geometry* Visual::release_geometry() {
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Geometry* temp = geometry_;
  geometry_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Geometry* Visual::unsafe_arena_release_geometry() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.geometry)
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Geometry* temp = geometry_;
  geometry_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Geometry* Visual::_internal_mutable_geometry() {
  _has_bits_[0] |= 0x00000008u;
  if (geometry_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Geometry>(GetArena());
    geometry_ = p;
  }
  return geometry_;
}
inline ::gazebo::msgs::Geometry* Visual::mutable_geometry() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.geometry)
  return _internal_mutable_geometry();
}
inline void Visual::set_allocated_geometry(::gazebo::msgs::Geometry* geometry) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(geometry_);
  }
  if (geometry) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(geometry)->GetArena();
    if (message_arena != submessage_arena) {
      geometry = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, geometry, submessage_arena);
    }
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  geometry_ = geometry;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.geometry)
}

// optional .gazebo.msgs.Material material = 10;
inline bool Visual::_internal_has_material() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  PROTOBUF_ASSUME(!value || material_ != nullptr);
  return value;
}
inline bool Visual::has_material() const {
  return _internal_has_material();
}
inline const ::gazebo::msgs::Material& Visual::_internal_material() const {
  const ::gazebo::msgs::Material* p = material_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Material&>(
      ::gazebo::msgs::_Material_default_instance_);
}
inline const ::gazebo::msgs::Material& Visual::material() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.material)
  return _internal_material();
}
inline void Visual::unsafe_arena_set_allocated_material(
    ::gazebo::msgs::Material* material) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(material_);
  }
  material_ = material;
  if (material) {
    _has_bits_[0] |= 0x00000010u;
  } else {
    _has_bits_[0] &= ~0x00000010u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Visual.material)
}
inline ::gazebo::msgs::Material* Visual::release_material() {
  _has_bits_[0] &= ~0x00000010u;
  ::gazebo::msgs::Material* temp = material_;
  material_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Material* Visual::unsafe_arena_release_material() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.material)
  _has_bits_[0] &= ~0x00000010u;
  ::gazebo::msgs::Material* temp = material_;
  material_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Material* Visual::_internal_mutable_material() {
  _has_bits_[0] |= 0x00000010u;
  if (material_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Material>(GetArena());
    material_ = p;
  }
  return material_;
}
inline ::gazebo::msgs::Material* Visual::mutable_material() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.material)
  return _internal_mutable_material();
}
inline void Visual::set_allocated_material(::gazebo::msgs::Material* material) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(material_);
  }
  if (material) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(material)->GetArena();
    if (message_arena != submessage_arena) {
      material = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, material, submessage_arena);
    }
    _has_bits_[0] |= 0x00000010u;
  } else {
    _has_bits_[0] &= ~0x00000010u;
  }
  material_ = material;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.material)
}

// optional bool visible = 11;
inline bool Visual::_internal_has_visible() const {
  bool value = (_has_bits_[0] & 0x00001000u) != 0;
  return value;
}
inline bool Visual::has_visible() const {
  return _internal_has_visible();
}
inline void Visual::clear_visible() {
  visible_ = false;
  _has_bits_[0] &= ~0x00001000u;
}
inline bool Visual::_internal_visible() const {
  return visible_;
}
inline bool Visual::visible() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.visible)
  return _internal_visible();
}
inline void Visual::_internal_set_visible(bool value) {
  _has_bits_[0] |= 0x00001000u;
  visible_ = value;
}
inline void Visual::set_visible(bool value) {
  _internal_set_visible(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.visible)
}

// optional bool delete_me = 12;
inline bool Visual::_internal_has_delete_me() const {
  bool value = (_has_bits_[0] & 0x00002000u) != 0;
  return value;
}
inline bool Visual::has_delete_me() const {
  return _internal_has_delete_me();
}
inline void Visual::clear_delete_me() {
  delete_me_ = false;
  _has_bits_[0] &= ~0x00002000u;
}
inline bool Visual::_internal_delete_me() const {
  return delete_me_;
}
inline bool Visual::delete_me() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.delete_me)
  return _internal_delete_me();
}
inline void Visual::_internal_set_delete_me(bool value) {
  _has_bits_[0] |= 0x00002000u;
  delete_me_ = value;
}
inline void Visual::set_delete_me(bool value) {
  _internal_set_delete_me(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.delete_me)
}

// optional bool is_static = 13;
inline bool Visual::_internal_has_is_static() const {
  bool value = (_has_bits_[0] & 0x00004000u) != 0;
  return value;
}
inline bool Visual::has_is_static() const {
  return _internal_has_is_static();
}
inline void Visual::clear_is_static() {
  is_static_ = false;
  _has_bits_[0] &= ~0x00004000u;
}
inline bool Visual::_internal_is_static() const {
  return is_static_;
}
inline bool Visual::is_static() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.is_static)
  return _internal_is_static();
}
inline void Visual::_internal_set_is_static(bool value) {
  _has_bits_[0] |= 0x00004000u;
  is_static_ = value;
}
inline void Visual::set_is_static(bool value) {
  _internal_set_is_static(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.is_static)
}

// repeated .gazebo.msgs.Plugin plugin = 14;
inline int Visual::_internal_plugin_size() const {
  return plugin_.size();
}
inline int Visual::plugin_size() const {
  return _internal_plugin_size();
}
inline ::gazebo::msgs::Plugin* Visual::mutable_plugin(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.plugin)
  return plugin_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Plugin >*
Visual::mutable_plugin() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.Visual.plugin)
  return &plugin_;
}
inline const ::gazebo::msgs::Plugin& Visual::_internal_plugin(int index) const {
  return plugin_.Get(index);
}
inline const ::gazebo::msgs::Plugin& Visual::plugin(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.plugin)
  return _internal_plugin(index);
}
inline ::gazebo::msgs::Plugin* Visual::_internal_add_plugin() {
  return plugin_.Add();
}
inline ::gazebo::msgs::Plugin* Visual::add_plugin() {
  // @@protoc_insertion_point(field_add:gazebo.msgs.Visual.plugin)
  return _internal_add_plugin();
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Plugin >&
Visual::plugin() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.Visual.plugin)
  return plugin_;
}

// optional .gazebo.msgs.Vector3d scale = 15;
inline bool Visual::_internal_has_scale() const {
  bool value = (_has_bits_[0] & 0x00000020u) != 0;
  PROTOBUF_ASSUME(!value || scale_ != nullptr);
  return value;
}
inline bool Visual::has_scale() const {
  return _internal_has_scale();
}
inline const ::gazebo::msgs::Vector3d& Visual::_internal_scale() const {
  const ::gazebo::msgs::Vector3d* p = scale_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Vector3d&>(
      ::gazebo::msgs::_Vector3d_default_instance_);
}
inline const ::gazebo::msgs::Vector3d& Visual::scale() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.scale)
  return _internal_scale();
}
inline void Visual::unsafe_arena_set_allocated_scale(
    ::gazebo::msgs::Vector3d* scale) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(scale_);
  }
  scale_ = scale;
  if (scale) {
    _has_bits_[0] |= 0x00000020u;
  } else {
    _has_bits_[0] &= ~0x00000020u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Visual.scale)
}
inline ::gazebo::msgs::Vector3d* Visual::release_scale() {
  _has_bits_[0] &= ~0x00000020u;
  ::gazebo::msgs::Vector3d* temp = scale_;
  scale_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Vector3d* Visual::unsafe_arena_release_scale() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.scale)
  _has_bits_[0] &= ~0x00000020u;
  ::gazebo::msgs::Vector3d* temp = scale_;
  scale_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Vector3d* Visual::_internal_mutable_scale() {
  _has_bits_[0] |= 0x00000020u;
  if (scale_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Vector3d>(GetArena());
    scale_ = p;
  }
  return scale_;
}
inline ::gazebo::msgs::Vector3d* Visual::mutable_scale() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.scale)
  return _internal_mutable_scale();
}
inline void Visual::set_allocated_scale(::gazebo::msgs::Vector3d* scale) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(scale_);
  }
  if (scale) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(scale)->GetArena();
    if (message_arena != submessage_arena) {
      scale = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, scale, submessage_arena);
    }
    _has_bits_[0] |= 0x00000020u;
  } else {
    _has_bits_[0] &= ~0x00000020u;
  }
  scale_ = scale;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.scale)
}

// optional .gazebo.msgs.Visual.Meta meta = 16;
inline bool Visual::_internal_has_meta() const {
  bool value = (_has_bits_[0] & 0x00000040u) != 0;
  PROTOBUF_ASSUME(!value || meta_ != nullptr);
  return value;
}
inline bool Visual::has_meta() const {
  return _internal_has_meta();
}
inline void Visual::clear_meta() {
  if (meta_ != nullptr) meta_->Clear();
  _has_bits_[0] &= ~0x00000040u;
}
inline const ::gazebo::msgs::Visual_Meta& Visual::_internal_meta() const {
  const ::gazebo::msgs::Visual_Meta* p = meta_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Visual_Meta&>(
      ::gazebo::msgs::_Visual_Meta_default_instance_);
}
inline const ::gazebo::msgs::Visual_Meta& Visual::meta() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.meta)
  return _internal_meta();
}
inline void Visual::unsafe_arena_set_allocated_meta(
    ::gazebo::msgs::Visual_Meta* meta) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(meta_);
  }
  meta_ = meta;
  if (meta) {
    _has_bits_[0] |= 0x00000040u;
  } else {
    _has_bits_[0] &= ~0x00000040u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Visual.meta)
}
inline ::gazebo::msgs::Visual_Meta* Visual::release_meta() {
  _has_bits_[0] &= ~0x00000040u;
  ::gazebo::msgs::Visual_Meta* temp = meta_;
  meta_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Visual_Meta* Visual::unsafe_arena_release_meta() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Visual.meta)
  _has_bits_[0] &= ~0x00000040u;
  ::gazebo::msgs::Visual_Meta* temp = meta_;
  meta_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Visual_Meta* Visual::_internal_mutable_meta() {
  _has_bits_[0] |= 0x00000040u;
  if (meta_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Visual_Meta>(GetArena());
    meta_ = p;
  }
  return meta_;
}
inline ::gazebo::msgs::Visual_Meta* Visual::mutable_meta() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Visual.meta)
  return _internal_mutable_meta();
}
inline void Visual::set_allocated_meta(::gazebo::msgs::Visual_Meta* meta) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete meta_;
  }
  if (meta) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::GetArena(meta);
    if (message_arena != submessage_arena) {
      meta = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, meta, submessage_arena);
    }
    _has_bits_[0] |= 0x00000040u;
  } else {
    _has_bits_[0] &= ~0x00000040u;
  }
  meta_ = meta;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Visual.meta)
}

// optional .gazebo.msgs.Visual.Type type = 17;
inline bool Visual::_internal_has_type() const {
  bool value = (_has_bits_[0] & 0x00008000u) != 0;
  return value;
}
inline bool Visual::has_type() const {
  return _internal_has_type();
}
inline void Visual::clear_type() {
  type_ = 0;
  _has_bits_[0] &= ~0x00008000u;
}
inline ::gazebo::msgs::Visual_Type Visual::_internal_type() const {
  return static_cast< ::gazebo::msgs::Visual_Type >(type_);
}
inline ::gazebo::msgs::Visual_Type Visual::type() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Visual.type)
  return _internal_type();
}
inline void Visual::_internal_set_type(::gazebo::msgs::Visual_Type value) {
  assert(::gazebo::msgs::Visual_Type_IsValid(value));
  _has_bits_[0] |= 0x00008000u;
  type_ = value;
}
inline void Visual::set_type(::gazebo::msgs::Visual_Type value) {
  _internal_set_type(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Visual.type)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------


typedef boost::shared_ptr<gazebo::msgs::Visual> VisualPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

PROTOBUF_NAMESPACE_OPEN

template <> struct is_proto_enum< ::gazebo::msgs::Visual_Type> : ::std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor< ::gazebo::msgs::Visual_Type>() {
  return ::gazebo::msgs::Visual_Type_descriptor();
}

PROTOBUF_NAMESPACE_CLOSE

typedef const boost::shared_ptr<gazebo::msgs::Visual const> ConstVisualPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_visual_2eproto
