// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: projector.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_projector_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_projector_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3016000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3016000 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "pose.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_projector_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_projector_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxiliaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[1]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_projector_2eproto;
namespace gazebo {
namespace msgs {
class Projector;
struct ProjectorDefaultTypeInternal;
GZ_MSGS_VISIBLE extern ProjectorDefaultTypeInternal _Projector_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::Projector* Arena::CreateMaybeMessage<::gazebo::msgs::Projector>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

// ===================================================================

class GZ_MSGS_VISIBLE Projector PROTOBUF_FINAL :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.Projector) */ {
 public:
  inline Projector() : Projector(nullptr) {}
  ~Projector() override;
  explicit constexpr Projector(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Projector(const Projector& from);
  Projector(Projector&& from) noexcept
    : Projector() {
    *this = ::std::move(from);
  }

  inline Projector& operator=(const Projector& from) {
    CopyFrom(from);
    return *this;
  }
  inline Projector& operator=(Projector&& from) noexcept {
    if (GetArena() == from.GetArena()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Projector& default_instance() {
    return *internal_default_instance();
  }
  static inline const Projector* internal_default_instance() {
    return reinterpret_cast<const Projector*>(
               &_Projector_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Projector& a, Projector& b) {
    a.Swap(&b);
  }
  inline void Swap(Projector* other) {
    if (other == this) return;
    if (GetArena() == other->GetArena()) {
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Projector* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline Projector* New() const final {
    return CreateMaybeMessage<Projector>(nullptr);
  }

  Projector* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<Projector>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const Projector& from);
  void MergeFrom(const Projector& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  ::PROTOBUF_NAMESPACE_ID::uint8* _InternalSerialize(
      ::PROTOBUF_NAMESPACE_ID::uint8* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Projector* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.Projector";
  }
  protected:
  explicit Projector(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  private:
  static void ArenaDtor(void* object);
  inline void RegisterArenaDtor(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kNameFieldNumber = 1,
    kTextureFieldNumber = 2,
    kPoseFieldNumber = 3,
    kEnabledFieldNumber = 7,
    kFovFieldNumber = 4,
    kNearClipFieldNumber = 5,
    kFarClipFieldNumber = 6,
  };
  // required string name = 1;
  bool has_name() const;
  private:
  bool _internal_has_name() const;
  public:
  void clear_name();
  const std::string& name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_name();
  std::string* release_name();
  void set_allocated_name(std::string* name);
  private:
  const std::string& _internal_name() const;
  void _internal_set_name(const std::string& value);
  std::string* _internal_mutable_name();
  public:

  // optional string texture = 2;
  bool has_texture() const;
  private:
  bool _internal_has_texture() const;
  public:
  void clear_texture();
  const std::string& texture() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_texture(ArgT0&& arg0, ArgT... args);
  std::string* mutable_texture();
  std::string* release_texture();
  void set_allocated_texture(std::string* texture);
  private:
  const std::string& _internal_texture() const;
  void _internal_set_texture(const std::string& value);
  std::string* _internal_mutable_texture();
  public:

  // optional .gazebo.msgs.Pose pose = 3;
  bool has_pose() const;
  private:
  bool _internal_has_pose() const;
  public:
  void clear_pose();
  const ::gazebo::msgs::Pose& pose() const;
  ::gazebo::msgs::Pose* release_pose();
  ::gazebo::msgs::Pose* mutable_pose();
  void set_allocated_pose(::gazebo::msgs::Pose* pose);
  private:
  const ::gazebo::msgs::Pose& _internal_pose() const;
  ::gazebo::msgs::Pose* _internal_mutable_pose();
  public:
  void unsafe_arena_set_allocated_pose(
      ::gazebo::msgs::Pose* pose);
  ::gazebo::msgs::Pose* unsafe_arena_release_pose();

  // optional bool enabled = 7 [default = true];
  bool has_enabled() const;
  private:
  bool _internal_has_enabled() const;
  public:
  void clear_enabled();
  bool enabled() const;
  void set_enabled(bool value);
  private:
  bool _internal_enabled() const;
  void _internal_set_enabled(bool value);
  public:

  // optional double fov = 4 [default = 0.785];
  bool has_fov() const;
  private:
  bool _internal_has_fov() const;
  public:
  void clear_fov();
  double fov() const;
  void set_fov(double value);
  private:
  double _internal_fov() const;
  void _internal_set_fov(double value);
  public:

  // optional double near_clip = 5 [default = 0.1];
  bool has_near_clip() const;
  private:
  bool _internal_has_near_clip() const;
  public:
  void clear_near_clip();
  double near_clip() const;
  void set_near_clip(double value);
  private:
  double _internal_near_clip() const;
  void _internal_set_near_clip(double value);
  public:

  // optional double far_clip = 6 [default = 10];
  bool has_far_clip() const;
  private:
  bool _internal_has_far_clip() const;
  public:
  void clear_far_clip();
  double far_clip() const;
  void set_far_clip(double value);
  private:
  double _internal_far_clip() const;
  void _internal_set_far_clip(double value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.Projector)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr name_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr texture_;
  ::gazebo::msgs::Pose* pose_;
  bool enabled_;
  double fov_;
  double near_clip_;
  double far_clip_;
  friend struct ::TableStruct_projector_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Projector

// required string name = 1;
inline bool Projector::_internal_has_name() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Projector::has_name() const {
  return _internal_has_name();
}
inline void Projector::clear_name() {
  name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& Projector::name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.name)
  return _internal_name();
}
template <typename ArgT0, typename... ArgT>
PROTOBUF_ALWAYS_INLINE
inline void Projector::set_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArena());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Projector.name)
}
inline std::string* Projector::mutable_name() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Projector.name)
  return _internal_mutable_name();
}
inline const std::string& Projector::_internal_name() const {
  return name_.Get();
}
inline void Projector::_internal_set_name(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  name_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArena());
}
inline std::string* Projector::_internal_mutable_name() {
  _has_bits_[0] |= 0x00000001u;
  return name_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArena());
}
inline std::string* Projector::release_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Projector.name)
  if (!_internal_has_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  return name_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline void Projector::set_allocated_name(std::string* name) {
  if (name != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  name_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), name,
      GetArena());
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Projector.name)
}

// optional string texture = 2;
inline bool Projector::_internal_has_texture() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool Projector::has_texture() const {
  return _internal_has_texture();
}
inline void Projector::clear_texture() {
  texture_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000002u;
}
inline const std::string& Projector::texture() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.texture)
  return _internal_texture();
}
template <typename ArgT0, typename... ArgT>
PROTOBUF_ALWAYS_INLINE
inline void Projector::set_texture(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000002u;
 texture_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, static_cast<ArgT0 &&>(arg0), args..., GetArena());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Projector.texture)
}
inline std::string* Projector::mutable_texture() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Projector.texture)
  return _internal_mutable_texture();
}
inline const std::string& Projector::_internal_texture() const {
  return texture_.Get();
}
inline void Projector::_internal_set_texture(const std::string& value) {
  _has_bits_[0] |= 0x00000002u;
  texture_.Set(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, value, GetArena());
}
inline std::string* Projector::_internal_mutable_texture() {
  _has_bits_[0] |= 0x00000002u;
  return texture_.Mutable(::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::EmptyDefault{}, GetArena());
}
inline std::string* Projector::release_texture() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Projector.texture)
  if (!_internal_has_texture()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000002u;
  return texture_.ReleaseNonDefault(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), GetArena());
}
inline void Projector::set_allocated_texture(std::string* texture) {
  if (texture != nullptr) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  texture_.SetAllocated(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), texture,
      GetArena());
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Projector.texture)
}

// optional .gazebo.msgs.Pose pose = 3;
inline bool Projector::_internal_has_pose() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || pose_ != nullptr);
  return value;
}
inline bool Projector::has_pose() const {
  return _internal_has_pose();
}
inline const ::gazebo::msgs::Pose& Projector::_internal_pose() const {
  const ::gazebo::msgs::Pose* p = pose_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Pose&>(
      ::gazebo::msgs::_Pose_default_instance_);
}
inline const ::gazebo::msgs::Pose& Projector::pose() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.pose)
  return _internal_pose();
}
inline void Projector::unsafe_arena_set_allocated_pose(
    ::gazebo::msgs::Pose* pose) {
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  pose_ = pose;
  if (pose) {
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Projector.pose)
}
inline ::gazebo::msgs::Pose* Projector::release_pose() {
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
  if (GetArena() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
  return temp;
}
inline ::gazebo::msgs::Pose* Projector::unsafe_arena_release_pose() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Projector.pose)
  _has_bits_[0] &= ~0x00000004u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Pose* Projector::_internal_mutable_pose() {
  _has_bits_[0] |= 0x00000004u;
  if (pose_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Pose>(GetArena());
    pose_ = p;
  }
  return pose_;
}
inline ::gazebo::msgs::Pose* Projector::mutable_pose() {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Projector.pose)
  return _internal_mutable_pose();
}
inline void Projector::set_allocated_pose(::gazebo::msgs::Pose* pose) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArena();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  if (pose) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose)->GetArena();
    if (message_arena != submessage_arena) {
      pose = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, pose, submessage_arena);
    }
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  pose_ = pose;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Projector.pose)
}

// optional double fov = 4 [default = 0.785];
inline bool Projector::_internal_has_fov() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool Projector::has_fov() const {
  return _internal_has_fov();
}
inline void Projector::clear_fov() {
  fov_ = 0.785;
  _has_bits_[0] &= ~0x00000010u;
}
inline double Projector::_internal_fov() const {
  return fov_;
}
inline double Projector::fov() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.fov)
  return _internal_fov();
}
inline void Projector::_internal_set_fov(double value) {
  _has_bits_[0] |= 0x00000010u;
  fov_ = value;
}
inline void Projector::set_fov(double value) {
  _internal_set_fov(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Projector.fov)
}

// optional double near_clip = 5 [default = 0.1];
inline bool Projector::_internal_has_near_clip() const {
  bool value = (_has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool Projector::has_near_clip() const {
  return _internal_has_near_clip();
}
inline void Projector::clear_near_clip() {
  near_clip_ = 0.1;
  _has_bits_[0] &= ~0x00000020u;
}
inline double Projector::_internal_near_clip() const {
  return near_clip_;
}
inline double Projector::near_clip() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.near_clip)
  return _internal_near_clip();
}
inline void Projector::_internal_set_near_clip(double value) {
  _has_bits_[0] |= 0x00000020u;
  near_clip_ = value;
}
inline void Projector::set_near_clip(double value) {
  _internal_set_near_clip(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Projector.near_clip)
}

// optional double far_clip = 6 [default = 10];
inline bool Projector::_internal_has_far_clip() const {
  bool value = (_has_bits_[0] & 0x00000040u) != 0;
  return value;
}
inline bool Projector::has_far_clip() const {
  return _internal_has_far_clip();
}
inline void Projector::clear_far_clip() {
  far_clip_ = 10;
  _has_bits_[0] &= ~0x00000040u;
}
inline double Projector::_internal_far_clip() const {
  return far_clip_;
}
inline double Projector::far_clip() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.far_clip)
  return _internal_far_clip();
}
inline void Projector::_internal_set_far_clip(double value) {
  _has_bits_[0] |= 0x00000040u;
  far_clip_ = value;
}
inline void Projector::set_far_clip(double value) {
  _internal_set_far_clip(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Projector.far_clip)
}

// optional bool enabled = 7 [default = true];
inline bool Projector::_internal_has_enabled() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool Projector::has_enabled() const {
  return _internal_has_enabled();
}
inline void Projector::clear_enabled() {
  enabled_ = true;
  _has_bits_[0] &= ~0x00000008u;
}
inline bool Projector::_internal_enabled() const {
  return enabled_;
}
inline bool Projector::enabled() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Projector.enabled)
  return _internal_enabled();
}
inline void Projector::_internal_set_enabled(bool value) {
  _has_bits_[0] |= 0x00000008u;
  enabled_ = value;
}
inline void Projector::set_enabled(bool value) {
  _internal_set_enabled(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Projector.enabled)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

typedef boost::shared_ptr<gazebo::msgs::Projector> ProjectorPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

typedef const boost::shared_ptr<gazebo::msgs::Projector const> ConstProjectorPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_projector_2eproto
