syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface ImageStamped
/// \brief Message for an image with a time


import "time.proto";
import "image.proto";

message ImageStamped
{
  // Time when the data was captured
  required Time time          = 1;
  required Image image        = 2;
}
