syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface ModelConfiguration 
/// \brief Message for model configuration (joint positions)


import "time.proto";
import "pose.proto";

message ModelConfiguration
{
  required Time time                        = 1; // Time when the pose should be enforced
  repeated string joint_names               = 2;
  repeated double joint_positions           = 3;
  optional Pose pose                        = 4; // Specify model pose
  optional string link_name                 = 5; // Option to set model pose by specifying pose of link
}

