syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Publish
/// \brief Message that contains information about a publisher of data


message Publish
{
  required string topic    = 1;
  required string msg_type = 2;
  required string host     = 3;
  required uint32 port     = 4;
}
